from setuptools import setup

setup(
    name='',
    version='0.0.1',
    description='{{description}}',
    author='{{author}}',
    author_email='{{email}}',
    download_url='',
    license='MIT',
    install_requires=[],
    setup_requires=['green'],
    keywords=['snips'],
    include_package_data=True,
    packages=[
        '{{project_name}}'
    ]
)